-- -------------------------------------------------------------------------- --
--                              GT100GCX by kunda                             --
-- -------------------------------------------------------------------------- --
--                                                                            --
--              ---===[ Gnomish Top100 GUID Catcher Xtrem ]===---             --
--                                                                            --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- GT100GCX scans the combatlog, mouseovered units, your group/raid and the   --
--                chat and catches all character GUIDs (plus date, class and  --
--                faction).                                                   --
--                All scans are optional and can turned off (ALL ENABLED by   --
--                DEFAULT).                                                   --
--                                                                            --
-- GT100GCX has a GUI accessible via Interface Option Menu or Slash command   --
--              to view the saved data.                                       --
--                                                                            --
-- Q: What is a GUID?                                                         --
-- A: GUID stands for Globally Unique IDentifier. In WoW a GUID is a 64-bit   --
--    hexadecimal string. The GUID is a simple number in the order characters --
--    are created. An older character has a lower number than a new.          --
--    Since Patch 3.1 every WoW character has a real globally unique GUID     --
--    because a battlegroup identifier was added to every character GUID!     --
--    The character GUID is NOT available via armory.                         --
--                                                                            --
-- Q: So, why save GUIDs?                                                     --
-- A: Because you should know the real freaks on your realm!? Hmmm...         --
--    The only purpose of this addon is to find old characters.               --
--    Yes, it's useless. So what?                                             --
--                                                                            --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- Notes:                                                                     --
-- - searchquery will only search in player names                             --
-- - date format: YYYY-MM-DD                                                  --
-- - slash commands: /gcx - /gt100gcx                                         --
--          example: /gcx         : open GT100GCX GUI                         --
--                   /gcx options : open GT100GCX InterfaceOptionsFrame       --
--                                                                            --
-- -------------------------------------------------------------------------- --
--                                                                            --
--                               FOR GNOMEREGAN                               --
--                                                                            --
-- -------------------------------------------------------------------------- --

-- ---------------------------------------------------------------------------------------------------------------------
GT100GCX_Data = {}                       -- SavedVariable data table
GT100GCX_Options = {}                    -- SavedVariable options table

local _G                  = _G
local UnitGUID            = _G.UnitGUID
local UnitName            = _G.UnitName
local UnitClass           = _G.UnitClass
local UnitFactionGroup    = _G.UnitFactionGroup
local UnitIsPlayer        = _G.UnitIsPlayer
local UnitIsUnit          = _G.UnitIsUnit
local GetPlayerInfoByGUID = _G.GetPlayerInfoByGUID
local GetNumRaidMembers   = _G.GetNumRaidMembers
local GetNumPartyMembers  = _G.GetNumPartyMembers
local bit_band            = _G.bit.band
local math_floor          = _G.math.floor
local string_find         = _G.string.find
local string_lower        = _G.string.lower
local string_sub          = _G.string.sub
local table_sort          = _G.table.sort
local table_wipe          = _G.table.wipe

local GT100GCX = CreateFrame("Frame")    -- event container
local L = GT100GCX_Locales               -- localization table

local playerRealm = GetCVar("realmName") -- current realm name
local playerName = UnitName("player")    -- current player name
local playerClass                        -- current player class
local playerFaction                      -- current player faction

local inCombat = false                   -- false = out of combat | true = in combat
local rosterhack = 0                     -- execute RAID_ROSTER_UPDATE block only if GetNumRaidMembers() has changed

local GUItab = {}                        -- main data table
local GUItabCount                        -- main data table count
local GUItabSortBy = 1                   -- main data table sort by: 1 = pos/GUID | 2 = name | 3 = faction | 4 = class | 5 = realm | 6 = date
local GUItabSortOrder = 1                -- main data table sort order: 0 = asc | 1 = desc
local GUItabRealm = playerRealm          -- for GUI: Realm (default: current realm name)
local GUIRealmToDel                      -- for GUI: selected Realm to delete

local GUIStatusTable = {}                -- for GUI Status: table with class/faction count

local GUIRealmTable = {}                 -- realm data table
local GUIRealmTableCount                 -- realm data table count

local SearchQuery                        -- GUI search query

local color = { -- e.g.: color.Gold.hex | color.Orange.rgb[1]...[3]
	LightYellow = {hex = "ffff7f", rgb = {1,    1,    0.49 }},
	Orange      = {hex = "ff9900", rgb = {1,    0.6,  0    }},
	Grey        = {hex = "cccccc", rgb = {0.8,  0.8,  0.8  }},
	White       = {hex = "ffffff", rgb = {1,    1,    1    }},
	Gold        = {hex = "ffd100", rgb = {1,    0.82, 0    }},
	Rselect     = {                rgb = {1,    1,    1,   0.35 }}
}

local classcolors = RAID_CLASS_COLORS
local classimg = "Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes"
local classes = {
	DRUID       = {number =  1, icon = {194/256,249/256,  5/256, 60/256}},
	HUNTER      = {number =  2, icon = {  5/256, 60/256, 69/256,124/256}},
	MAGE        = {number =  3, icon = { 68/256,123/256,  5/256, 60/256}},
	PALADIN     = {number =  4, icon = {  5/256, 60/256,133/256,188/256}},
	PRIEST      = {number =  5, icon = {131/256,186/256, 69/256,124/256}},
	ROGUE       = {number =  6, icon = {131/256,186/256,  5/256, 60/256}},
	SHAMAN      = {number =  7, icon = { 68/256,123/256, 69/256,124/256}},
	WARLOCK     = {number =  8, icon = {194/256,249/256, 69/256,124/256}},
	WARRIOR     = {number =  9, icon = {  5/256, 60/256,  5/256, 60/256}},
	DEATHKNIGHT = {number = 10, icon = { 68/256,123/256,133/256,188/256}}
}
local classesINT = {
	[1]  = "DRUID",
	[2]  = "HUNTER",
	[3]  = "MAGE",
	[4]  = "PALADIN",
	[5]  = "PRIEST",
	[6]  = "ROGUE",
	[7]  = "SHAMAN",
	[8]  = "WARLOCK",
	[9]  = "WARRIOR",
	[10] = "DEATHKNIGHT"
}

local factionImageAlliance = "Interface\\TargetingFrame\\UI-PVP-Alliance" -- coords (XML) { 3/64,39/64, 2/64,38/64}
local factionImageHorde    = "Interface\\TargetingFrame\\UI-PVP-Horde"    -- coords (XML) { 3/64,39/64, 2/64,38/64}
local factions = {
	Alliance = {number = 1, color = "00adf0", short = string_sub(FACTION_ALLIANCE, 0, 1)},
	Horde    = {number = 2, color = "ff1a1a", short = string_sub(FACTION_HORDE, 0, 1)}
}
local factionsINT = {
	[1] = "Alliance",
	[2] = "Horde"
}
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function Print(...)
	print("|cff"..color.LightYellow.hex.."GT100GCX:|r", ...)
end

local function ClassHexColor(class)
	local hex = string.format("%.2x%.2x%.2x", classcolors[class].r*255, classcolors[class].g*255, classcolors[class].b*255)
	return hex or "cccccc"
end

local function SlashHandler(msg)
	msg = strtrim(string_lower(msg or ""))
	if msg == "options" then
		InterfaceOptionsFrame_OpenToCategory("GT100GCX")
	else
		GT100GCX_FrameToggle()
	end
end

-- -------------------------------------------------------------------------- --
-- Check for the new GUID format (new in 3.1) START                           --
-- -------------------------------------------------------------------------- --
-- example: old character GUID: 0x00000000000365C8                            --
--          new character GUID: 0x04800000000365C8                            --
-- The '048' after '0x' seems to be some kind of battlegroup identification   --
-- and was added because of the new cross-language battlegrounds.             --
-- -------------------------------------------------------------------------- --
local function newGUIDcheck(realm, guid)
	--local x1 = GetTime() --TIMEcheck
	if GT100GCX_Data.realmID and GT100GCX_Data.realmID[realm] then return end

	local realmID = string_sub(guid, 0, 5)

	-- playerdata START
	if GT100GCX_Data.playerdata and GT100GCX_Data.playerdata[realm] then

		local temp = {}
		local tempNum
		local x = 1

		for k, v in pairs(GT100GCX_Data.playerdata[realm]) do
			local realmGUID = string_sub(k, 0, 5)
			local charGUID  = string_sub(k, 6)
			temp[x] = {}
			temp[x].charGUID  = charGUID
			temp[x].realmGUID = realmGUID
			temp[x].data_d    = v.d
			temp[x].data_fc   = v.fc
			temp[x].data_n    = v.n
			temp[x].delThis   = false
			temp[x].check     = true
			if realmGUID == "0x000" then
				temp[x].check   = false
			end
			x = x + 1
		end
		tempNum = #temp
		--Print("tempNum="..tempNum)	
		
		for i = 1, tempNum do
			if temp[i].check then
				for k, v in pairs(GT100GCX_Data.playerdata[realm]) do
					local realmGUID = string_sub(k, 0, 5)
					local charGUID  = string_sub(k, 6)
					if temp[i].charGUID == charGUID and temp[i].realmGUID ~= realmGUID then
						temp[i].delThis = true
						--Print("DEL "..k.."-"..charGUID)
					end
				end
			end
		end

		GT100GCX_Data.playerdata[realm] = nil
		GT100GCX_Data.playerdata[realm] = {}

		for i = 1, tempNum do
			if not temp[i].delThis then
				local newguid = realmID..temp[i].charGUID
				GT100GCX_Data.playerdata[realm][newguid] = {}
				GT100GCX_Data.playerdata[realm][newguid].d  = temp[i].data_d
				GT100GCX_Data.playerdata[realm][newguid].fc = temp[i].data_fc
				GT100GCX_Data.playerdata[realm][newguid].n  = temp[i].data_n
				--Print("NEW "..newguid.."-"..temp[i].data_n.."-"..temp[i].data_d.."-"..temp[i].data_fc)
			end
		end

		table_wipe(temp)
	end
	-- playerdata END

	-- data START
	if GT100GCX_Data.data and GT100GCX_Data.data[realm] then

		local temp = {}
		local tempNum
		local x = 1

		for k, v in pairs(GT100GCX_Data.data[realm]) do
			local realmGUID = string_sub(k, 0, 5)
			local charGUID  = string_sub(k, 6)
			temp[x] = {}
			temp[x].charGUID  = charGUID
			temp[x].realmGUID = realmGUID
			temp[x].data_d    = v.d
			temp[x].data_fc   = v.fc
			temp[x].data_n    = v.n
			temp[x].delThis   = false
			temp[x].check     = true  -- new
			if realmGUID == "0x000" then
				temp[x].check   = false -- old
			end
			x = x + 1
		end
		tempNum = #temp
		--Print("tempNum="..tempNum)	
		
		for i = 1, tempNum do
			if temp[i].check then
				for k, v in pairs(GT100GCX_Data.data[realm]) do
					local realmGUID = string_sub(k, 0, 5)
					local charGUID  = string_sub(k, 6)
					if temp[i].charGUID == charGUID and temp[i].realmGUID ~= realmGUID then
						temp[i].delThis = true
						--Print("TO DEL "..k.."-"..charGUID)
					end
				end
			end
		end

		GT100GCX_Data.data[realm] = nil
		GT100GCX_Data.data[realm] = {}

		for i = 1, tempNum do
			if not temp[i].delThis then
				local newguid = realmID..temp[i].charGUID
				GT100GCX_Data.data[realm][newguid] = {}
				GT100GCX_Data.data[realm][newguid].d  = temp[i].data_d
				GT100GCX_Data.data[realm][newguid].fc = temp[i].data_fc
				GT100GCX_Data.data[realm][newguid].n  = temp[i].data_n
				--Print("NEW "..newguid.."-"..temp[i].data_n.."-"..temp[i].data_d.."-"..temp[i].data_fc)
			end
		end

		table_wipe(temp)
	end
	-- data END

	-- maxGUID START
	if GT100GCX_Data.maxGUID and GT100GCX_Data.maxGUID[realm] then
		local temp = {}
		local x = 1
		for k, v in pairs(GT100GCX_Data.data[realm]) do
			temp[x] = k
			x = x + 1
		end
		
		table_sort(temp, function(a, b) return a<b end)
		
		GT100GCX_Data.maxGUID[realm] = temp[#temp]
		--Print("maxGUID ("..realm..") "..temp[#temp])
		table_wipe(temp)
	end
	-- maxGUID END

	-- maxEntry START
	if GT100GCX_Data.maxEntry and GT100GCX_Data.maxEntry[realm] then
		local x = 0
		for k, v in pairs(GT100GCX_Data.data[realm]) do
			x = x + 1
		end

		GT100GCX_Data.maxEntry[realm] = x
		--Print("maxEntry ("..realm..") "..x)
	end
	-- maxEntry END

	-- realmID START
	if not GT100GCX_Data.realmID then GT100GCX_Data.realmID = {} end
	GT100GCX_Data.realmID[realm] = realmID
	-- realmID END
	--local x3 = GetTime() Print("TIMEcheck: "..x3-x1) --TIMEcheck
end
-- -------------------------------------------------------------------------- --
-- Check for the new GUID format (new in 3.1) END                             --
-- -------------------------------------------------------------------------- --
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function GT100GCX_InitOptions()
	SlashCmdList["GT100GCX"] = SlashHandler
	SLASH_GT100GCX1 = "/gcx"
	SLASH_GT100GCX2 = "/gt100gcx"

	-- DB version check START
	if GT100GCX_Options.version == nil then
		GT100GCX_Options.version = 1.3
	end
	if GT100GCX_Options.version < 1.2 then
		GT100GCX_Options.version = 1.2  -- DB version
		GT100GCX_Options.chatall = nil  -- removed in 1.2

		local function newDate(date)
			local dateY = string_sub(date,0,4)
			local dateM = string_sub(date,5,6)
			local dateD = string_sub(date,7,8)
			return dateY.."-"..dateM.."-"..dateD
		end
		
		-- update all data to new date format
		if GT100GCX_Data.data then
			for k,v in pairs(GT100GCX_Data.data) do
				for k2,v2 in pairs(GT100GCX_Data.data[k]) do
					GT100GCX_Data.data[k][k2].d = newDate(v2.d)
				end
			end
		end
  	
		-- update all playerdata to new date format
		if GT100GCX_Data.playerdata then
			for k,v in pairs(GT100GCX_Data.playerdata) do
				for k2,v2 in pairs(GT100GCX_Data.playerdata[k]) do
					GT100GCX_Data.playerdata[k][k2].d = newDate(v2.d)
				end
			end
		end
	end
	if GT100GCX_Options.version < 1.3 then
		GT100GCX_Options.version  = 1.3 -- DB version
		GT100GCX_Data.battlegroup = nil -- removed in 1.3
	end
	-- DB version check END

	if GT100GCX_Options.chatnew == nil       then GT100GCX_Options.chatnew       = false end -- show new entries in chat
	if GT100GCX_Options.maxdata == nil       then GT100GCX_Options.maxdata       = 100   end -- maximum saved data entries per realm
	if GT100GCX_Options.tooltipinfo == nil   then GT100GCX_Options.tooltipinfo   = false end -- tooltip display
	if GT100GCX_Options.combatlogscan == nil then GT100GCX_Options.combatlogscan = true  end -- combatlog scan
	if GT100GCX_Options.mouseoverscan == nil then GT100GCX_Options.mouseoverscan = true  end -- mouseover scan
	if GT100GCX_Options.groupscan == nil     then GT100GCX_Options.groupscan     = true  end -- group scan
	if GT100GCX_Options.chatscan == nil      then GT100GCX_Options.chatscan      = true  end -- chat scan
	if GT100GCX_Options.incombatscan == nil  then GT100GCX_Options.incombatscan  = false end -- combat / out-of-combat
	if GT100GCX_Options.realmscan == nil     then GT100GCX_Options.realmscan     = true  end -- scan only playerRealm / scan all realms 

	if not GT100GCX_Options.combatlogscan then
		GT100GCX:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end

	if not GT100GCX_Options.groupscan then
		GT100GCX:UnregisterEvent("PARTY_MEMBERS_CHANGED")
		GT100GCX:UnregisterEvent("RAID_ROSTER_UPDATE")
	end

	if not GT100GCX_Options.chatscan then
		GT100GCX:UnregisterEvent("CHAT_MSG_CHANNEL")
		GT100GCX:UnregisterEvent("CHAT_MSG_CHANNEL_NOTICE_USER")
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function GT100GCX_LDBcheck()
	if LibStub and LibStub:GetLibrary("CallbackHandler-1.0", true) and LibStub:GetLibrary("LibDataBroker-1.1", true) then
		LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("GT100GCX", {
			type = "launcher",
			icon = "Interface\\GossipFrame\\GossipGossipIcon",
			OnClick = function(self, button)
				GT100GCX_FrameToggle()
			end,
		})
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function GT100GCX_CreateInterfaceOptions()
	local options = CreateFrame("FRAME", "GT100GCX_InterfaceOptions")
	options.name = "GT100GCX"

	local title = options:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	title:SetText("GT100GCX")
	title:SetJustifyH("LEFT")
	title:SetJustifyV("TOP")
	title:SetPoint("TOPLEFT", 16, -16)

	local subText = options:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	subText:SetText("Gnomish Top100 GUID Catcher Xtrem")
	subText:SetNonSpaceWrap(true)
	subText:SetJustifyH("LEFT")
	subText:SetJustifyV("TOP")
	subText:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)

	local GUI = CreateFrame("Button", "GT100GCX_InterfaceOptions_GUI", GT100GCX_InterfaceOptions, "UIPanelButtonTemplate")
	GT100GCX_InterfaceOptions_GUI:SetPoint("TOPLEFT", subText, "BOTTOMLEFT", 0, -10)
	GT100GCX_InterfaceOptions_GUI:SetWidth(120)
	GT100GCX_InterfaceOptions_GUI:SetHeight(24)
	GT100GCX_InterfaceOptions_GUIText:SetText(L["Open GUI"])
	GT100GCX_InterfaceOptions_GUI:SetScript("OnClick", function(self)
		InterfaceOptionsFrame_Show() -- hide InterfaceOptionsFrame
		HideUIPanel(GameMenuFrame)   -- hide GameMenuFrame
		GT100GCX_FrameToggle()       -- show GT100GCX GUI
	end)

	local slashcommandText = options:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	slashcommandText:SetText("(|cff"..color.LightYellow.hex.."/gcx|r - |cff"..color.LightYellow.hex.."/gt100gcx|r)")
	slashcommandText:SetNonSpaceWrap(true)
	slashcommandText:SetPoint("LEFT", GUI, "RIGHT", 10, 0)

	local titleScan = options:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	titleScan:SetText(L["Scan Options"])
	titleScan:SetNonSpaceWrap(true)
	titleScan:SetJustifyH("LEFT")
	titleScan:SetJustifyV("TOP")
	titleScan:SetPoint("TOPLEFT", GUI, "BOTTOMLEFT", 0, -10)

	local CombatLogScan = CreateFrame("CheckButton", "GT100GCX_InterfaceOptions_CombatLogScan", GT100GCX_InterfaceOptions, "InterfaceOptionsCheckButtonTemplate")
	GT100GCX_InterfaceOptions_CombatLogScan:SetPoint("TOPLEFT", titleScan, "TOPLEFT", 10, -16)
	GT100GCX_InterfaceOptions_CombatLogScanText:SetText(L["CombatLog Scan"])
	GT100GCX_InterfaceOptions_CombatLogScan:SetChecked(GT100GCX_Options.combatlogscan)
	GT100GCX_InterfaceOptions_CombatLogScan:SetScript("OnClick", function(self)
		GT100GCX_Options.combatlogscan = not GT100GCX_Options.combatlogscan
		if GT100GCX_Options.combatlogscan then
			GT100GCX:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		else
			GT100GCX:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		end
	end)

	local MouseOverScan = CreateFrame("CheckButton", "GT100GCX_InterfaceOptions_MouseOverScan", GT100GCX_InterfaceOptions, "InterfaceOptionsCheckButtonTemplate")
	GT100GCX_InterfaceOptions_MouseOverScan:SetPoint("TOPLEFT", CombatLogScan, "TOPLEFT", 0, -20)
	GT100GCX_InterfaceOptions_MouseOverScanText:SetText(L["MouseOver Scan"])
	GT100GCX_InterfaceOptions_MouseOverScan:SetChecked(GT100GCX_Options.mouseoverscan)
	GT100GCX_InterfaceOptions_MouseOverScan:SetScript("OnClick", function(self)
		GT100GCX_Options.mouseoverscan = not GT100GCX_Options.mouseoverscan
	end)

	local GroupScan = CreateFrame("CheckButton", "GT100GCX_InterfaceOptions_GroupScan", GT100GCX_InterfaceOptions, "InterfaceOptionsCheckButtonTemplate")
	GT100GCX_InterfaceOptions_GroupScan:SetPoint("TOPLEFT", MouseOverScan, "TOPLEFT", 0, -20)
	GT100GCX_InterfaceOptions_GroupScanText:SetText(L["Group Scan"])
	GT100GCX_InterfaceOptions_GroupScan:SetChecked(GT100GCX_Options.groupscan)
	GT100GCX_InterfaceOptions_GroupScan:SetScript("OnClick", function(self)
		GT100GCX_Options.groupscan = not GT100GCX_Options.groupscan
		if GT100GCX_Options.groupscan then
			GT100GCX:RegisterEvent("PARTY_MEMBERS_CHANGED")
			GT100GCX:RegisterEvent("RAID_ROSTER_UPDATE")
		else
			GT100GCX:UnregisterEvent("PARTY_MEMBERS_CHANGED")
			GT100GCX:UnregisterEvent("RAID_ROSTER_UPDATE")
		end
	end)

	local ChatScan = CreateFrame("CheckButton", "GT100GCX_InterfaceOptions_ChatScan", GT100GCX_InterfaceOptions, "InterfaceOptionsCheckButtonTemplate")
	GT100GCX_InterfaceOptions_ChatScan:SetPoint("TOPLEFT", GroupScan, "TOPLEFT", 0, -20)
	GT100GCX_InterfaceOptions_ChatScanText:SetText(L["Chat Scan"])
	GT100GCX_InterfaceOptions_ChatScan:SetChecked(GT100GCX_Options.chatscan)
	GT100GCX_InterfaceOptions_ChatScan:SetScript("OnClick", function(self)
		GT100GCX_Options.chatscan = not GT100GCX_Options.chatscan
		if GT100GCX_Options.chatscan then
			GT100GCX:RegisterEvent("CHAT_MSG_CHANNEL")
			GT100GCX:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE_USER")
		else
			GT100GCX:UnregisterEvent("CHAT_MSG_CHANNEL")
			GT100GCX:UnregisterEvent("CHAT_MSG_CHANNEL_NOTICE_USER")
		end
	end)

	local InCombatScan = CreateFrame("CheckButton", "GT100GCX_InterfaceOptions_InCombatScan", GT100GCX_InterfaceOptions, "InterfaceOptionsCheckButtonTemplate")
	GT100GCX_InterfaceOptions_InCombatScan:SetPoint("TOPLEFT", ChatScan, "TOPLEFT", 0, -30)
	GT100GCX_InterfaceOptions_InCombatScanText:SetText(L["Scan in combat"])
	GT100GCX_InterfaceOptions_InCombatScan:SetChecked(GT100GCX_Options.incombatscan)
	GT100GCX_InterfaceOptions_InCombatScan:SetScript("OnClick", function(self)
		GT100GCX_Options.incombatscan = not GT100GCX_Options.incombatscan
		if inCombat then
			if GT100GCX_Options.incombatscan then
				if GT100GCX_Options.combatlogscan then
					GT100GCX:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
				end
				if GT100GCX_Options.groupscan then
					GT100GCX:RegisterEvent("PARTY_MEMBERS_CHANGED")
					GT100GCX:RegisterEvent("RAID_ROSTER_UPDATE")
				end
				if GT100GCX_Options.chatscan then
					GT100GCX:RegisterEvent("CHAT_MSG_CHANNEL")
					GT100GCX:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE_USER")
				end
			else
				GT100GCX:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
				GT100GCX:UnregisterEvent("RAID_ROSTER_UPDATE")
				GT100GCX:UnregisterEvent("PARTY_MEMBERS_CHANGED")
				GT100GCX:UnregisterEvent("CHAT_MSG_CHANNEL")
				GT100GCX:UnregisterEvent("CHAT_MSG_CHANNEL_NOTICE_USER")
			end
		end
	end)
	GT100GCX_InterfaceOptions_InCombatScan.tooltipText = L["|cffff2020!!! WARNING !!!|r\nIt's not recommended to enable this option! Only use this option if you know what you are doing!!!"]

	local RealmScan = CreateFrame("CheckButton", "GT100GCX_InterfaceOptions_RealmScan", GT100GCX_InterfaceOptions, "InterfaceOptionsCheckButtonTemplate")
	GT100GCX_InterfaceOptions_RealmScan:SetPoint("TOPLEFT", InCombatScan, "TOPLEFT", 0, -20)
	GT100GCX_InterfaceOptions_RealmScanText:SetText(L["Scan ONLY your realm"])
	GT100GCX_InterfaceOptions_RealmScan:SetChecked(GT100GCX_Options.realmscan)
	GT100GCX_InterfaceOptions_RealmScan:SetScript("OnClick", function(self)
		GT100GCX_Options.realmscan = not GT100GCX_Options.realmscan
	end)
	GT100GCX_InterfaceOptions_RealmScan.tooltipText = L["Enabled: scan only your realm\nDisabled: scan any realm"]

	local titleDataEntries = options:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	titleDataEntries:SetText(L["Maximum data entries per server:"])
	titleDataEntries:SetNonSpaceWrap(true)
	titleDataEntries:SetJustifyH("LEFT")
	titleDataEntries:SetJustifyV("TOP")
	titleDataEntries:SetPoint("TOPLEFT", RealmScan, "BOTTOMLEFT", 0, -6)

	local DataEntries100 = CreateFrame("CheckButton", "GT100GCX_InterfaceOptions_DataEntries100", GT100GCX_InterfaceOptions, "InterfaceOptionsCheckButtonTemplate")
	GT100GCX_InterfaceOptions_DataEntries100:SetPoint("TOPLEFT", titleDataEntries, "TOPLEFT", 0, -20)
	GT100GCX_InterfaceOptions_DataEntries100Text:SetText("100")
	if GT100GCX_Options.maxdata == 100 then
		GT100GCX_InterfaceOptions_DataEntries100:SetChecked(true)
	else
		GT100GCX_InterfaceOptions_DataEntries100:SetChecked(false)
	end
	GT100GCX_InterfaceOptions_DataEntries100:SetScript("OnClick", function(self)
		GT100GCX_InterfaceOptions_DataEntries100:SetChecked(true)
		GT100GCX_InterfaceOptions_DataEntries1000:SetChecked(false)
		if GT100GCX_Options.maxdata == 1000 then
			GT100GCX_ConfirmPopup(L["|cffff2020!!! WARNING !!!|r\n\nThis action DELETES all saved GUIDs with a realm position greater 100 from all realms!!!\n\nDo you really want to do this?"], GT100GCX_DeleteDBdata)
		end
	end)

	local DataEntries1000 = CreateFrame("CheckButton", "GT100GCX_InterfaceOptions_DataEntries1000", GT100GCX_InterfaceOptions, "InterfaceOptionsCheckButtonTemplate")
	GT100GCX_InterfaceOptions_DataEntries1000:SetPoint("TOPLEFT", DataEntries100, "TOPLEFT", 0, -20)
	GT100GCX_InterfaceOptions_DataEntries1000Text:SetText("1000")
	if GT100GCX_Options.maxdata == 1000 then
		GT100GCX_InterfaceOptions_DataEntries1000:SetChecked(true)
	else
		GT100GCX_InterfaceOptions_DataEntries1000:SetChecked(false)
	end
	GT100GCX_InterfaceOptions_DataEntries1000:SetScript("OnClick", function(self)
		GT100GCX_Options.maxdata = 1000
		GT100GCX_InterfaceOptions_DataEntries100:SetChecked(false)
		GT100GCX_InterfaceOptions_DataEntries1000:SetChecked(true)
	end)

	local titleOutput = options:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	titleOutput:SetText(L["Output"])
	titleOutput:SetNonSpaceWrap(true)
	titleOutput:SetJustifyH("LEFT")
	titleOutput:SetJustifyV("TOP")
	titleOutput:SetPoint("TOPLEFT", DataEntries1000, "BOTTOMLEFT", -10, -10)

	local ChatNewEntry = CreateFrame("CheckButton", "GT100GCX_InterfaceOptions_ChatNewEntry", GT100GCX_InterfaceOptions, "InterfaceOptionsCheckButtonTemplate")
	GT100GCX_InterfaceOptions_ChatNewEntry:SetPoint("TOPLEFT", titleOutput, "TOPLEFT", 10, -16)
	GT100GCX_InterfaceOptions_ChatNewEntryText:SetText(L["Show new entries"])
	GT100GCX_InterfaceOptions_ChatNewEntry:SetChecked(GT100GCX_Options.chatnew)
	GT100GCX_InterfaceOptions_ChatNewEntry:SetScript("OnClick", function(self)
		GT100GCX_Options.chatnew = not GT100GCX_Options.chatnew
	end)

	local TooltipInfo = CreateFrame("CheckButton", "GT100GCX_InterfaceOptions_TooltipInfo", GT100GCX_InterfaceOptions, "InterfaceOptionsCheckButtonTemplate")
	GT100GCX_InterfaceOptions_TooltipInfo:SetPoint("TOPLEFT", ChatNewEntry, "TOPLEFT", 0, -20)
	GT100GCX_InterfaceOptions_TooltipInfoText:SetText(L["Tooltip"])
	GT100GCX_InterfaceOptions_TooltipInfo:SetChecked(GT100GCX_Options.tooltipinfo)
	GT100GCX_InterfaceOptions_TooltipInfo:SetScript("OnClick", function(self)
		GT100GCX_Options.tooltipinfo = not GT100GCX_Options.tooltipinfo
	end)

	InterfaceOptions_AddCategory(options)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function GT100GCX_ConfirmPopup(message, func, ...)
	if not StaticPopupDialogs["GT100GCX_CONFIRM_DIALOG"] then
		StaticPopupDialogs["GT100GCX_CONFIRM_DIALOG"] = {}
	end
	local t = StaticPopupDialogs["GT100GCX_CONFIRM_DIALOG"]
	for k in pairs(t) do
		t[k] = nil
	end
	t.text = message
	t.button1 = ACCEPT or "Accept"
	t.button2 = CANCEL or "Cancel"
	t.OnAccept = function()
		-- temporary save scan options
		local temp_combatlogscan = GT100GCX_Options.combatlogscan
		local temp_mouseoverscan = GT100GCX_Options.mouseoverscan
		local temp_groupscan     = GT100GCX_Options.groupscan
		local temp_chatscan      = GT100GCX_Options.chatscan

		-- disable scan
		GT100GCX_Options.combatlogscan = false
		GT100GCX_Options.mouseoverscan = false
		GT100GCX_Options.groupscan     = false
		GT100GCX_Options.chatscan      = false

		GT100GCX_Options.maxdata = 100

		-- delete
		func(unpack(t))

		-- set back scan options
		GT100GCX_Options.combatlogscan = temp_combatlogscan
		GT100GCX_Options.mouseoverscan = temp_mouseoverscan
		GT100GCX_Options.groupscan     = temp_groupscan
		GT100GCX_Options.chatscan      = temp_chatscan
		
		if GT100GCXFrame:IsShown() then
			GT100GCX_UpdateAllLists()
		end
	end
	t.OnHide = function() 
		-- set Interface Options Buttons
		if GT100GCX_Options.maxdata == 100 then
			GT100GCX_InterfaceOptions_DataEntries100:SetChecked(true)
			GT100GCX_InterfaceOptions_DataEntries1000:SetChecked(false)
		else
			GT100GCX_InterfaceOptions_DataEntries100:SetChecked(false)
			GT100GCX_InterfaceOptions_DataEntries1000:SetChecked(true)
		end
	end

	for i = 1, select('#', ...) do
		t[i] = select(i, ...)
	end
	t.timeout = 0
	t.whileDead = 1
	t.hideOnEscape = 1

	StaticPopup_Show("GT100GCX_CONFIRM_DIALOG")
end

function GT100GCX_ConfirmDelRealmPopup(message, func, ...)
	if not StaticPopupDialogs["GT100GCX_CONFIRM_DEL_REALM_DIALOG"] then
		StaticPopupDialogs["GT100GCX_CONFIRM_DEL_REALM_DIALOG"] = {}
	end
	local t = StaticPopupDialogs["GT100GCX_CONFIRM_DEL_REALM_DIALOG"]
	for k in pairs(t) do
		t[k] = nil
	end
	t.text = message
	t.button1 = ACCEPT or "Accept"
	t.button2 = CANCEL or "Cancel"
	t.OnAccept = function()
		-- temporary save scan options
		local temp_combatlogscan = GT100GCX_Options.combatlogscan
		local temp_mouseoverscan = GT100GCX_Options.mouseoverscan
		local temp_groupscan     = GT100GCX_Options.groupscan
		local temp_chatscan      = GT100GCX_Options.chatscan

		-- disable scan
		GT100GCX_Options.combatlogscan = false
		GT100GCX_Options.mouseoverscan = false
		GT100GCX_Options.groupscan     = false
		GT100GCX_Options.chatscan      = false

		-- delete
		func(unpack(t))

		-- set back scan options
		GT100GCX_Options.combatlogscan = temp_combatlogscan
		GT100GCX_Options.mouseoverscan = temp_mouseoverscan
		GT100GCX_Options.groupscan     = temp_groupscan
		GT100GCX_Options.chatscan      = temp_chatscan
		if GT100GCXFrame:IsShown() then
			GT100GCX_UpdateAllLists()
		end
	end
	t.OnHide = function() 
		GT100GCXFrame_RealmOptions:Hide()
	end

	for i = 1, select('#', ...) do
		t[i] = select(i, ...)
	end
	t.timeout = 0
	t.whileDead = 1
	t.hideOnEscape = 1

	StaticPopup_Show("GT100GCX_CONFIRM_DEL_REALM_DIALOG")
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function GT100GCX_DeleteDBdata()
	local delsum
	-- iterate through all realms and delete entries greater than maxdata
	for k,v in pairs(GT100GCX_Data.data) do
		if GT100GCX_Data.maxEntry[k] > GT100GCX_Options.maxdata then
			-- create temp table for sort
			local temp = {}
			local x = 1
			for k2,v2 in pairs(GT100GCX_Data.data[k]) do
				temp[x] = k2
				x = x + 1
			end
			local tempCount = #temp

			-- sort temp table
			table_sort(temp, function(a, b) return a<b end)

			-- delete entries > maxdata
			local del = 0
			for i=(GT100GCX_Options.maxdata+1), tempCount do
				GT100GCX_Data.data[k][temp[i]] = nil
				del = del + 1
				delsum = true
			end

			-- set maxEntry & maxGUID
			GT100GCX_Data.maxEntry[k] = GT100GCX_Options.maxdata
			GT100GCX_Data.maxGUID[k] = temp[GT100GCX_Options.maxdata]

			if del > 1 then
				Print(L["Realm"].." |cff"..color.LightYellow.hex..k.."|r: "..del.." "..L["data entries deleted."])
			else
				Print(L["Realm"].." |cff"..color.LightYellow.hex..k.."|r: "..del.." "..L["data entry deleted."])
			end
			
			-- clean temp db
			table_wipe(temp)
		end
	end
	if not delsum then
		Print("0 "..L["data entries deleted."])
	end
end

function GT100GCX_DeleteRealmData()
	if not GUIRealmToDel then return end

	-- playerdata
	if GT100GCX_Data.playerdata and GT100GCX_Data.playerdata[GUIRealmToDel] then
		GT100GCX_Data.playerdata[GUIRealmToDel] = nil
	end	
	-- data
	if GT100GCX_Data.data and GT100GCX_Data.data[GUIRealmToDel] then
		GT100GCX_Data.data[GUIRealmToDel] = nil
	end
	-- maxGUID
	if GT100GCX_Data.maxGUID and GT100GCX_Data.maxGUID[GUIRealmToDel] then
		GT100GCX_Data.maxGUID[GUIRealmToDel] = nil
	end
	-- maxEntry
	if GT100GCX_Data.maxEntry and GT100GCX_Data.maxEntry[GUIRealmToDel] then
		GT100GCX_Data.maxEntry[GUIRealmToDel] = nil
	end
	-- realmID
	if GT100GCX_Data.realmID and GT100GCX_Data.realmID[GUIRealmToDel] then
		GT100GCX_Data.realmID[GUIRealmToDel] = nil
	end

	Print(L["Realm"].." |cff"..color.LightYellow.hex..GUIRealmToDel.."|r "..L["deleted"]..".")

	if GUIRealmToDel == playerRealm then
		GT100GCX_SavePlayerData()
	end

	if GUItabRealm == GUIRealmToDel then
		GUItabRealm = playerRealm
	end

	GUIRealmToDel = nil
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- unit mouseover hook
function GT100GCX_OnTooltipSetUnit()
	if not GT100GCX_Options.mouseoverscan and not GT100GCX_Options.tooltipinfo then return end
	if inCombat and not GT100GCX_Options.incombatscan                          then return end
	if not UnitIsPlayer("mouseover")                                           then return end
	if UnitIsUnit("player", "mouseover")                                       then return end

	local name, realm = UnitName("mouseover")
	local guid = UnitGUID("mouseover")

	-- mouseover scan
	if GT100GCX_Options.mouseoverscan then
		GT100GCX_SaveData(L["MOUSEOVER"], name, realm, guid)
	end
	-- tooltipinfo
	if GT100GCX_Options.tooltipinfo then
		-- check incoming data
		if not name then return end
		if name == UNKNOWNOBJECT then return end
		if realm == "" then return end
		if not realm then realm = playerRealm end
		if name == playerName and realm == playerRealm then return end
		if not guid then return end

		-- better and faster visual recognition (especially if position > maxdata)
		-- example: guid=0x04800000011F8796 -> guidPlain=11F8796 guidBase=7 -> in tooltip: 7 - 11F8796
		--          guid=0x0480000000017D32 -> guidPlain=17D32   guidBase=5 -> in tooltip: 5 - 17D32
		-- guidPlain = plain character GUID eg: without '0x', without realmID '048' and without leading zeros
		-- guidBase  = guidPlain length (the smaller the guidBase the older the char)
		local guidPlain, guidBase, s, e
		guidPlain = string_sub(guid, 6)
		s, e, guidPlain = string_find(guidPlain, "([1-9A-F].*)")
		s = s or 0
		e = e or 0
		guidPlain = guidPlain or "-"
		guidBase = e - s + 1
		if s == 0 and e == 0 then
			guidBase = "-"
		end

		if not GT100GCX_Data.data or not GT100GCX_Data.data[realm] or not GT100GCX_Data.data[realm][guid] then
			local str = ">"..GT100GCX_Options.maxdata
			if not GT100GCX_Options.mouseoverscan then
				str = "-"
			end
			GameTooltip:AddLine("|cff"..color.LightYellow.hex.."GT100GCX:|r |cff"..color.Orange.hex..L["Pos"]..": "..str.."|r |cff"..color.Grey.hex.."("..guidBase.." - "..guidPlain..")|r",1,1,1)
			GameTooltip:Show()
			return
		end

		-- create temp table for sort
		local temp = {}
		local x = 1
		for k,v in pairs(GT100GCX_Data.data[realm]) do
			temp[x] = k
			x = x + 1
		end
		local tempCount = #temp

		-- add playerdata to temp table for position
		if GT100GCX_Data.playerdata[realm] then
			local x = tempCount + 1
			for k,v in pairs(GT100GCX_Data.playerdata[realm]) do
				temp[x] = k
				x = x + 1
			end
			tempCount = #temp
		end

		-- sort temp table
		table_sort(temp, function(a, b) return a<b end)

		-- get guid position
		local guidPos
		for i=1, tempCount do
			if temp[i] == guid then
				guidPos = i
				break
			end
		end

		-- clean temp db
		table_wipe(temp)

		GameTooltip:AddLine("|cff"..color.LightYellow.hex.."GT100GCX:|r |cff"..color.Orange.hex..L["Pos"]..": "..guidPos.."|r |cff"..color.Grey.hex.."("..guidBase.." - "..guidPlain..")|r",1,1,1)
		GameTooltip:Show()
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- chat scan
function GT100GCX_ChatScan(event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12)
	-- arg2: name
	-- arg12: GUID
	if not GT100GCX_Options.chatscan                  then return end
	if inCombat and not GT100GCX_Options.incombatscan then return end

	if arg2 and arg12 and arg12 ~= "" then
		if event == "CHAT_MSG_CHANNEL_NOTICE_USER" then
			if arg1 ~= "OWNER_CHANGED" then return end -- Paranoida check...arg12 should be "" if not arg1==OWNER_CHANGED...
		end

		if string_find(arg2, "-", 1, true) then return end -- Paranoia check...the only chat we scan is from playerRealm...

		GT100GCX_SaveData(L["CHAT"], arg2, playerRealm, arg12)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- main scan data save function
function GT100GCX_SaveData(source, name, realm, guid)
	--local x1 = GetTime() --TIMEcheck
	-- check incoming data
	if not name then return end
	if name == UNKNOWNOBJECT then return end
	if realm == "" then return end
	if not realm then realm = playerRealm end
	if GT100GCX_Options.realmscan and realm ~= playerRealm then return end
	if name == playerName and realm == playerRealm then return end
	if not guid then return end

	-- WORKAROUND
	-- This check is necessary because the combatlog sourceGUID for buffs and debuffs that
	-- was applied before Patch 3.1 was not changed.
	-- If a character is logged-in the first time after the Patch 3.1-GUID-change and has buffs
	-- or debuffs from before 3.1 this combatlog sourceGUID starts with the wrong old format 0x000.
	-- Sad, but every combatlogscan needs a string.sub-check to filter out such old data :(
	if source == L["COMBATLOG"] then
		if string_sub(guid, 0, 5) == "0x000" then return end
	end
	-- WORKAROUND
	
	newGUIDcheck(realm, guid)
	--Print("TEST |cffcccccc"..source.." "..name.."-"..realm.." (guid="..guid..")|r") --TEST

	-- create data option table for realm
	if not GT100GCX_Data.data then GT100GCX_Data.data = {} end
	if not GT100GCX_Data.data[realm] then GT100GCX_Data.data[realm] = {} end

	-- create maxGUID option table for realm
	if not GT100GCX_Data.maxGUID then GT100GCX_Data.maxGUID = {} end
	if not GT100GCX_Data.maxGUID[realm] then GT100GCX_Data.maxGUID[realm] = guid end

	-- create maxEntry option table for realm
	if not GT100GCX_Data.maxEntry then GT100GCX_Data.maxEntry = {} end
	if not GT100GCX_Data.maxEntry[realm] then GT100GCX_Data.maxEntry[realm] = 0 end

	-- if guid greater than maxGUID[realm] return to sender
	if GT100GCX_Data.maxEntry[realm] == GT100GCX_Options.maxdata then
		if guid > GT100GCX_Data.maxGUID[realm] then return end
	end

	-- date
	local date = date("%Y-%m-%d")

	-- factionclass
	local faction
	local _, class, _, race = GetPlayerInfoByGUID(guid)
	if race == "NightElf" then
		faction = "Alliance"
	elseif race == "Human" then
		faction = "Alliance"
	elseif race == "Dwarf" then
		faction = "Alliance"
	elseif race == "Draenei" then
		faction = "Alliance"
	elseif race == "Gnome" then
		faction = "Alliance"
	else
		faction = "Horde"
	end

	local factionclass = 0
	local classcolor = color.Grey.hex
	local factiontxt = "" 
	if class and faction then
		factionclass = factions[faction].number + (classes[class].number / 100) -- e.g.: 1.03 = Alliance Mage, 2.02 = Horde Hunter, ...
		classcolor = ClassHexColor(class)
		factiontxt = "|cff"..factions[faction].color..factions[faction].short.."|r "
	end

	-- new guid?
	if not GT100GCX_Data.data[realm][guid] then
		-- save data to options table
		GT100GCX_Data.data[realm][guid] = {}
		GT100GCX_Data.data[realm][guid].d = date
		GT100GCX_Data.data[realm][guid].n = name
		GT100GCX_Data.data[realm][guid].fc = factionclass

		-- increment maxEntry
		if GT100GCX_Data.maxEntry[realm] < GT100GCX_Options.maxdata then
			GT100GCX_Data.maxEntry[realm] = GT100GCX_Data.maxEntry[realm] + 1
		end

		-- create temp table for sort
		local temp = {}
		local x = 1
		for k,v in pairs(GT100GCX_Data.data[realm]) do
			temp[x] = k
			x = x + 1
		end
		local tempCount = #temp

		-- sort temp table
		table_sort(temp, function(a, b) return a<b end)

		-- save highest guid
		if GT100GCX_Data.maxEntry[realm] < GT100GCX_Options.maxdata then
			GT100GCX_Data.maxGUID[realm] = temp[tempCount]
			--Print("highes guid (<maxData): .maxGUID[realm]"..temp[tempCount].." tempCount="..tempCount)
		else
			GT100GCX_Data.maxGUID[realm] = temp[GT100GCX_Options.maxdata]
			--Print("highes guid (=maxData): .maxGUID[realm]"..temp[GT100GCX_Options.maxdata].." tempCount="..GT100GCX_Options.maxdata)
		end

		-- delete guid that is not in our save range (GT100GCX_Options.maxdata)
		local deletedGUID
		if tempCount > GT100GCX_Options.maxdata then
			for i=(GT100GCX_Options.maxdata+1), tempCount do
				deletedGUID = temp[i]
				GT100GCX_Data.data[realm][deletedGUID] = nil
			end
		end

		-- was a new guid saved?
		if guid ~= deletedGUID then

			if GT100GCXFrame:IsShown() then
				GT100GCX_UpdateAllLists()
			end

			-- chat output
			if GT100GCX_Options.chatnew then
				-- add playerdata to temp table for position
				if GT100GCX_Data.playerdata[realm] then
					local x = tempCount + 1
					for k,v in pairs(GT100GCX_Data.playerdata[realm]) do
						temp[x] = k
						x = x + 1
					end
					tempCount = #temp

					-- sort temp table (with playerdata)
					table_sort(temp, function(a, b) return a<b end)
				end

				-- get guid position
				local guidPos
				for i=1, tempCount do
					if temp[i] == guid then
						guidPos = i
						break
					end
				end

				-- chat output
				Print("|cff"..color.Orange.hex..L["NEW"].." ("..source..")|r "..guidPos.." |cff"..classcolor..name.."|r-|cff"..color.LightYellow.hex..realm.."|r "..factiontxt.."|cff"..color.Grey.hex.."(guid="..guid..")|r")
			end
		end

		-- clean temp table
		table_wipe(temp)

	-- guid exists? do a name & factionclass check
	else
		GT100GCX_Data.data[realm][guid].d = date
		
		if GT100GCX_Data.data[realm][guid].n ~= name then
			if GT100GCX_Options.chatnew then
				Print("|cff"..color.Orange.hex..L["NAME CHANGED"].." ("..source..")|r "..L["oldname"].."=|cff"..classcolor..GT100GCX_Data.data[realm][guid].n.."|r "..L["newname"].."=|cff"..classcolor..name.."|r "..L["realm"].."=|cff"..color.LightYellow.hex..realm.."|r "..factiontxt.."|cff"..color.Grey.hex.."(guid="..guid..")|r")
			end
			GT100GCX_Data.data[realm][guid].n = name
		end
		if factionclass > 0 and (GT100GCX_Data.data[realm][guid].fc == 0 or GT100GCX_Data.data[realm][guid].fc ~= factionclass) then

			GT100GCX_Data.data[realm][guid].fc = factionclass

			if GT100GCX_Options.chatnew then
				-- create temp table for sort
				local temp = {}
				local x = 1
				for k,v in pairs(GT100GCX_Data.data[realm]) do
					temp[x] = k
					x = x + 1
				end
				local tempCount = #temp

				-- add playerdata to temp table for position
				if GT100GCX_Data.playerdata[realm] then
					local x = tempCount + 1
					for k,v in pairs(GT100GCX_Data.playerdata[realm]) do
						temp[x] = k
						x = x + 1
					end
					tempCount = #temp
				end

				-- sort temp table (with playerdata)
				table_sort(temp, function(a, b) return a<b end)

				-- get guid position
				local guidPos
				for i=1, tempCount do
					if temp[i] == guid then
						guidPos = i
						break
					end
				end

				-- clean temp table
				table_wipe(temp)

				Print("|cff"..color.Orange.hex..L["UPDATE"].." ("..source..")|r "..guidPos.." |cff"..classcolor..name.."|r-|cff"..color.LightYellow.hex..realm.."|r "..factiontxt.."|cff"..color.Grey.hex.."(guid="..guid..")|r")
			end
		end

		if GT100GCXFrame:IsShown() then
			GT100GCX_UpdateAllLists()
		end

	end
	--local x3 = GetTime() Print("TIMEcheck: "..x3-x1) --TIMEcheck
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- save player data
function GT100GCX_SavePlayerData()
	local guid = UnitGUID("player")
	if not guid then return end
	newGUIDcheck(playerRealm, guid)
	local date = date("%Y-%m-%d")
	local factionclass = factions[playerFaction].number + (classes[playerClass].number / 100)
	local classcolor = ClassHexColor(playerClass)
	local factiontxt = "|cff"..factions[playerFaction].color..factions[playerFaction].short.."|r "
	if not GT100GCX_Data.playerdata then GT100GCX_Data.playerdata = {} end
	if not GT100GCX_Data.playerdata[playerRealm] then GT100GCX_Data.playerdata[playerRealm] = {} end
	if not GT100GCX_Data.playerdata[playerRealm][guid] then
		GT100GCX_Data.playerdata[playerRealm][guid] = {}
		GT100GCX_Data.playerdata[playerRealm][guid].d = date
		GT100GCX_Data.playerdata[playerRealm][guid].n = playerName
		GT100GCX_Data.playerdata[playerRealm][guid].fc = factionclass
		if GT100GCX_Options.chatnew then
			Print("|cff"..color.Orange.hex..L["NEW PLAYERDATA"].."|r |cff"..classcolor..playerName.."|r-|cff"..color.LightYellow.hex..playerRealm.."|r "..factiontxt.."|cff"..color.Grey.hex.."(guid="..guid..")|r")
		end
	else
		GT100GCX_Data.playerdata[playerRealm][guid].d = date
		if GT100GCX_Data.playerdata[playerRealm][guid].n ~= playerName then
			if GT100GCX_Options.chatnew then
				Print("|cff"..color.Orange.hex..L["NAME CHANGED"].."|r "..L["oldname"].."=|cff"..classcolor..GT100GCX_Data.playerdata[playerRealm][guid].n.."|r "..L["newname"].."=|cff"..classcolor..playerName.."|r "..L["realm"].."=|cff"..color.LightYellow.hex..realm.."|r "..factiontxt.."|cff"..color.Grey.hex.."(guid="..guid..")|r")
			end
			GT100GCX_Data.playerdata[playerRealm][guid].n = playerName
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function GT100GCX_RealmOnClick(self, button)
	StaticPopup_Hide("GT100GCX_CONFIRM_DEL_REALM_DIALOG")
	GUIRealmToDel = nil
	if button == "RightButton" then
		GT100GCX_RealmOptions(self)
		return
	end
	GT100GCXFrame_RealmOptions:Hide()
	GT100GCX_SelectRealm(self:GetName())
end

function GT100GCX_RealmOptions(self)
	local name = self:GetName()
	local realm = _G[name.."Name"]:GetText()

	GT100GCXFrame_RealmOptionsText1Text:SetText(string.format(L["DELETE %s"], realm))
	local width = math.ceil(GT100GCXFrame_RealmOptionsText1Text:GetWidth())
	if width >= 120 then
		GT100GCXFrame_RealmOptions:SetWidth(width+40)
		GT100GCXFrame_RealmOptionsText1:SetWidth(width+40)
	else
		GT100GCXFrame_RealmOptions:SetWidth(150)
		GT100GCXFrame_RealmOptionsText1:SetWidth(150)
	end
	GT100GCXFrame_RealmOptionsText1Text:ClearAllPoints()
	GT100GCXFrame_RealmOptionsText1Text:SetPoint("LEFT", 25, 0)
	GT100GCXFrame_RealmOptions:SetPoint("TOPLEFT", name, "BOTTOMLEFT", 5, 0)
	GT100GCXFrame_RealmOptions:SetFrameLevel(GT100GCXFrame_groups:GetFrameLevel()+10)
	GT100GCXFrame_RealmOptions:Show()
	GUIRealmToDel = realm
end

function GT100GCX_DeleteRealm()
	if not GUIRealmToDel then return end
	GT100GCX_ConfirmDelRealmPopup(string.format(L["|cffff2020!!! WARNING !!!|r\n\nThis action DELETES all saved DATA from\nrealm |cffffff7f%s|r!!!\n\nDo you really want to do this?"], GUIRealmToDel), GT100GCX_DeleteRealmData)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function GT100GCX_SelectRealm(val)
	GUItabRealm = _G[val.."Name"]:GetText()

	for i=1, 25 do
		_G["GT100GCX_ListRealm"..i.."Texture"]:SetTexture(0, 0, 0, 0)
		_G["GT100GCX_ListRealm"..i.."Arrow"]:Hide()
	end
	_G[val.."Texture"]:SetTexture(color.Rselect.rgb[1], color.Rselect.rgb[2], color.Rselect.rgb[3], color.Rselect.rgb[4])
	_G[val.."Arrow"]:Show()

	GT100GCX_CreateGUItab()
	GT100GCX_FillList()
	GT100GCX_UpdateStatusFrame()
end

function GT100GCX_TableSort(sortBy)
	if GUItabSortBy == sortBy then
		if GUItabSortOrder == 1 then
			GUItabSortOrder = 0
		else
			GUItabSortOrder = 1
		end
	else
		GUItabSortOrder = 1
	end
	GUItabSortBy = sortBy
	GT100GCX_UpdateSortButtons()
	GT100GCX_CreateGUItab()
	GT100GCX_FillList()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function GT100GCX_CreateGUItab()
	--local x1 = GetTime() --TIMEcheck
	local function createGUIStatusTable(up, val)
		-- up 0 = faction
		-- up 1 = class
		-- up 2 = player
		if up == 0 and not val then
			val = "Zero"
		end
		if up == 1 and not val then
			return true
		end
		if up == 2 and not val then
			return true
		end
		--Print("val="..val)
		if not GUIStatusTable[val] then
			GUIStatusTable[val] = 1
		else
			GUIStatusTable[val] = GUIStatusTable[val] + 1
		end
	end

	table_wipe(GUIStatusTable)
	table_wipe(GUItab)

	-- -------------------------------------------------------------------------------------------------------------------
	-- no GT100GCX_Data.data at all (thats the case if only playerdata was saved)
	if not GT100GCX_Data.data then
		local x = 1
		for k,v in pairs(GT100GCX_Data.playerdata[playerRealm]) do
			GUItab[x] = {}
			GUItab[x].guid = k
			GUItab[x].name = "|cff"..color.Orange.hex..v.n.."|r"
			GUItab[x].namePlain = v.n
			GUItab[x].date = v.d
			GUItab[x].realm = playerRealm
			GUItab[x].faction = math_floor(v.fc) createGUIStatusTable(0, factionsINT[GUItab[x].faction])
			GUItab[x].class = (math_floor((v.fc*100)+0.5))-(math_floor(v.fc)*100) createGUIStatusTable(1, classesINT[GUItab[x].class])
			GUItab[x].player = 1 createGUIStatusTable(2, "player")
			x = x + 1
		end
		GUItabCount = #GUItab
	-- -------------------------------------------------------------------------------------------------------------------
	else
		-- create temp table
		local x = 1
		if GT100GCX_Data.data[GUItabRealm] then
			for k,v in pairs(GT100GCX_Data.data[GUItabRealm]) do
				GUItab[x] = {}
				GUItab[x].guid = k
				GUItab[x].name = "|cff"..color.White.hex..v.n.."|r"
				GUItab[x].namePlain = v.n
				GUItab[x].date = v.d
				GUItab[x].realm = GUItabRealm
				GUItab[x].faction = math_floor(v.fc) createGUIStatusTable(0, factionsINT[GUItab[x].faction])
				GUItab[x].class = (math_floor((v.fc*100)+0.5))-(math_floor(v.fc)*100) createGUIStatusTable(1, classesINT[GUItab[x].class])
				GUItab[x].player = 0
				x = x + 1
			end
			GUItabCount = #GUItab
		else
			GUItabCount = 0
		end
		-- add playerdata to temp table
		if GT100GCX_Data.playerdata[GUItabRealm] then
			x = GUItabCount + 1
			for k,v in pairs(GT100GCX_Data.playerdata[GUItabRealm]) do
				GUItab[x] = {}
				GUItab[x].guid = k
				GUItab[x].name = "|cff"..color.Orange.hex..v.n.."|r"
				GUItab[x].namePlain = v.n
				GUItab[x].date = v.d
				GUItab[x].realm = GUItabRealm
				GUItab[x].faction = math_floor(v.fc) createGUIStatusTable(0, factionsINT[GUItab[x].faction])
				GUItab[x].class = (math_floor((v.fc*100)+0.5))-(math_floor(v.fc)*100) createGUIStatusTable(1, classesINT[GUItab[x].class])
				GUItab[x].player = 1 createGUIStatusTable(2, "player")
				x = x + 1
			end
			GUItabCount = #GUItab
		end
		-- ---------------------------------------------------------------------------------------------------------------
	end


	table_sort(GUItab, function(a, b)
		if a.guid < b.guid then
			return true
		end
	end)

	-- add position
	for i=1, GUItabCount do
		GUItab[i].pos = i
	end

	-- sort temp table
	local sortfunc
	if GUItabSortBy == 1 then -- 1 = pos/GUID
		if GUItabSortOrder == 1 then
			sortfunc = function(a, b)
				if a.guid < b.guid then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.guid > b.guid then
					return true
				end
			end
		end
	elseif GUItabSortBy == 2 then -- 2 = name
		if GUItabSortOrder == 1 then
			sortfunc = function(a, b)
				if a.name < b.name then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.name > b.name then
					return true
				end
			end
		end
	elseif GUItabSortBy == 3 then -- 3 = faction
		if GUItabSortOrder == 1 then
			sortfunc = function(a, b)
				if a.faction == b.faction then
					if a.guid < b.guid then
						return true
					end
				end
				if a.faction < b.faction then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.faction == b.faction then
					if a.guid < b.guid then
						return true
					end
				end
				if a.faction > b.faction then
					return true
				end
			end
		end
	elseif GUItabSortBy == 4 then -- 4 = class
		if GUItabSortOrder == 1 then
			sortfunc = function(a, b)
				if a.class == b.class then
					if a.guid < b.guid then
						return true
					end
				end
				if a.class < b.class then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.class == b.class then
					if a.guid < b.guid then
						return true
					end
				end
				if a.class > b.class then
					return true
				end
			end
		end
	elseif GUItabSortBy == 5 then -- 5 = realm
		if GUItabSortOrder == 1 then
			sortfunc = function(a, b)
				if a.realm == b.realm then
					if a.guid < b.guid then
						return true
					end
				end
				if a.realm < b.realm then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.realm == b.realm then
					if a.guid < b.guid then
						return true
					end
				end
				if a.realm > b.realm then
					return true
				end
			end
		end
	elseif GUItabSortBy == 6 then -- 6 = date
		if GUItabSortOrder == 1 then
			sortfunc = function(a, b)
				if a.date == b.date then
					if a.guid < b.guid then
						return true
					end
				end
				if a.date < b.date then
					return true
				end
			end
		else
			sortfunc = function(a, b)
				if a.date == b.date then
					if a.guid < b.guid then
						return true
					end
				end
				if a.date > b.date then
					return true
				end
			end
		end
	end

	table_sort(GUItab, sortfunc)

	-- search
	if SearchQuery then
		table_wipe(GUIStatusTable)
		local newtemp = {}
		local x = 1
		local lowerSearchQuery = string_lower(SearchQuery)
		for i=1, GUItabCount do
			if string_find(string_lower(GUItab[i].namePlain), lowerSearchQuery, 1, true) then
				newtemp[x] = {}
				newtemp[x].pos       = GUItab[i].pos
				newtemp[x].guid      = GUItab[i].guid
				newtemp[x].name      = GUItab[i].name
				newtemp[x].namePlain = GUItab[i].namePlain
				newtemp[x].date      = GUItab[i].date
				newtemp[x].realm     = GUItab[i].realm
				newtemp[x].faction   = GUItab[i].faction createGUIStatusTable(0, factionsINT[newtemp[x].faction])
				newtemp[x].class     = GUItab[i].class createGUIStatusTable(1, classesINT[newtemp[x].class])
				newtemp[x].player    = GUItab[i].player if newtemp[x].player == 1 then createGUIStatusTable(2, "player") end
				x = x + 1
			end
		end

		-- fill GUItab with newtemp
		table_wipe(GUItab)
		for i=1, #newtemp do
			GUItab[i] = newtemp[i]
		end
		table_wipe(newtemp)
		GUItabCount = #GUItab
	end
	--local x3 = GetTime() Print("TIMEcheck: "..x3-x1) --TIMEcheck
end

function GT100GCX_OnVerticalScroll()
	GT100GCXFrame_ScrollFrameScrollBar:SetValue(arg1)
	GT100GCXFrame_ScrollFrame.offset = math_floor((arg1 / 12) + 0.5)
	GT100GCX_FillList()
end

function GT100GCX_FillList()
	--local x1 = GetTime() --TIMEcheck
	local showtableNumSize = 25

	if GUItabCount < showtableNumSize then
		for i=1, showtableNumSize do
			_G["GT100GCX_List"..i]:Hide()
		end
		showtableNumSize = GUItabCount
	end

	FauxScrollFrame_Update(GT100GCXFrame_ScrollFrame, GUItabCount, showtableNumSize, 12)

	if GUItabCount <= 25 then
		GT100GCXFrame_ScrollFrame:Hide()
	else
		GT100GCXFrame_ScrollFrame:Show()
	end

	-- fill list with data
	local j = 1
	local sliderPos = GT100GCXFrame_ScrollFrame.offset
	for i=sliderPos+1, sliderPos+showtableNumSize do
		if sliderPos < GUItabCount then
			local Pos     = _G["GT100GCX_List"..j.."Pos"]
			local Name    = _G["GT100GCX_List"..j.."Name"]
			local Faction = _G["GT100GCX_List"..j.."Faction"]
			local Class   = _G["GT100GCX_List"..j.."Class"]
			local Realm   = _G["GT100GCX_List"..j.."Realm"]
			local Guid    = _G["GT100GCX_List"..j.."Guid"]
			local Date    = _G["GT100GCX_List"..j.."Date"]

			Pos:SetTextColor(color.Gold.rgb[1], color.Gold.rgb[2], color.Gold.rgb[3], 1)
			Name:SetTextColor(color.White.rgb[1], color.White.rgb[2], color.White.rgb[3], 1)
			Realm:SetTextColor(color.LightYellow.rgb[1], color.LightYellow.rgb[2], color.LightYellow.rgb[3], 1)
			Guid:SetTextColor(color.Grey.rgb[1], color.Grey.rgb[2], color.Grey.rgb[3], 1)
			Date:SetTextColor(color.Grey.rgb[1], color.Grey.rgb[2], color.Grey.rgb[3], 1)
			if GUItab[i].player == 1 then
				Pos:SetTextColor(color.Orange.rgb[1], color.Orange.rgb[2], color.Orange.rgb[3], 1)
				Name:SetTextColor(color.Orange.rgb[1], color.Orange.rgb[2], color.Orange.rgb[3], 1)
				Realm:SetTextColor(color.Orange.rgb[1], color.Orange.rgb[2], color.Orange.rgb[3], 1)
				Guid:SetTextColor(color.Orange.rgb[1], color.Orange.rgb[2], color.Orange.rgb[3], 1)
				Date:SetTextColor(color.Orange.rgb[1], color.Orange.rgb[2], color.Orange.rgb[3], 1)
			end

			Pos:SetText( GUItab[i].pos )
			Name:SetText( GUItab[i].name )

			Faction:Hide()
			Class:SetTexCoord(0, 0, 0, 0)
			if GUItab[i].class > 0 then
				if GUItab[i].faction == 1 then
					Faction:SetTexture(factionImageAlliance)
				else
					Faction:SetTexture(factionImageHorde)
				end
				Faction:Show()
				local cx = classes[classesINT[GUItab[i].class]]
				Class:SetTexCoord(cx.icon[1], cx.icon[2], cx.icon[3], cx.icon[4])
			end

			Realm:SetText( GUItab[i].realm )
			Guid:SetText( GUItab[i].guid )
			Date:SetText( GUItab[i].date )
			_G["GT100GCX_List"..j]:Show()
			j = j + 1
		end
	end
	--local x3 = GetTime() Print("TIMEcheck: "..x3-x1) --TIMEcheck
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function GT100GCX_CreateGUIRealmTable()
	table_wipe(GUIRealmTable)

	local x = 1
	if not GT100GCX_Data.data then
		for k,v in pairs(GT100GCX_Data.playerdata) do
			GUIRealmTable[x] = k
			x = x + 1
		end
	else
		for k,v in pairs(GT100GCX_Data.data) do
			GUIRealmTable[x] = k
			x = x + 1
		end
		local check
		for k,v in pairs(GT100GCX_Data.playerdata) do
			check = 0
			for i=1, #GUIRealmTable do
				if GUIRealmTable[i] == k then
					check = check + 1
				end
			end
			if check == 0 then
				GUIRealmTable[x] = k
				x = x + 1
			end
		end
	end

	GUIRealmTableCount = #GUIRealmTable

	table_sort(GUIRealmTable, function(a, b)
		if a < b then
			return true
		end
	end)
end

function GT100GCX_OnVerticalScrollRealm()
	GT100GCXFrame_RealmScrollFrameScrollBar:SetValue(arg1)
	GT100GCXFrame_RealmScrollFrame.offset = math_floor((arg1 / 12) + 0.5)
	GT100GCX_FillRealmList()
end

function GT100GCX_FillRealmList()
	local showtableNumSize = 25

	if GUIRealmTableCount < showtableNumSize then
		for i=1, showtableNumSize do
			_G["GT100GCX_ListRealm"..i]:Hide()
		end
		showtableNumSize = GUIRealmTableCount
	end

	FauxScrollFrame_Update(GT100GCXFrame_RealmScrollFrame, GUIRealmTableCount, showtableNumSize, 12)

	if GUIRealmTableCount <= 25 then
		GT100GCXFrame_RealmScrollFrame:Hide()
	else
		GT100GCXFrame_RealmScrollFrame:Show()
	end

	-- fill list with data
	local j = 1
	local sliderPos = GT100GCXFrame_RealmScrollFrame.offset
	for i=sliderPos+1, sliderPos+showtableNumSize do
		if sliderPos < GUIRealmTableCount then
			_G["GT100GCX_ListRealm"..j.."Texture"]:SetTexture(0, 0, 0, 0)
			_G["GT100GCX_ListRealm"..j.."Arrow"]:Hide()
			if GUItabRealm == GUIRealmTable[i] then
				_G["GT100GCX_ListRealm"..j.."Texture"]:SetTexture(color.Rselect.rgb[1], color.Rselect.rgb[2], color.Rselect.rgb[3], color.Rselect.rgb[4])
				_G["GT100GCX_ListRealm"..j.."Arrow"]:Show()
			end
			_G["GT100GCX_ListRealm"..j.."Name"]:SetText( GUIRealmTable[i] )
			_G["GT100GCX_ListRealm"..j]:Show()
			j = j + 1
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function GT100GCX_UpdateAllLists()
	GT100GCX_CreateGUItab()
	GT100GCX_FillList()
	GT100GCX_CreateGUIRealmTable()
	GT100GCX_FillRealmList()
	GT100GCX_UpdateStatusFrame()
end

function GT100GCX_FrameToggle()
	GT100GCXFrame_RealmOptions:Hide()
	if GT100GCXFrame:IsShown() then
		GT100GCXFrame:Hide()
	else
		GT100GCXFrame:Show()
	end
end

function GT100GCX_MainFramePos()
	if GT100GCX_Options.relP then
		GT100GCXFrame:ClearAllPoints()
		GT100GCXFrame:SetPoint(GT100GCX_Options.relP, GT100GCX_Options.posX, GT100GCX_Options.posY)
	end
end

function GT100GCX_MainFrameHide()
	PlaySound("igQuestListClose")
	table_wipe(GUItab)
	GT100GCX_InterfaceOptions_GUI:Enable()
end

function GT100GCX_MainFrameShow()
	PlaySound("igQuestListOpen")
	GT100GCX_InterfaceOptions_GUI:Disable()
	GT100GCX_MainFramePos()
	GT100GCX_UpdateAllLists()
end

function GT100GCX_SaveFramePos()
	local point, relativeTo, relativePoint, x, y = GT100GCXFrame:GetPoint()
	GT100GCX_Options.posX = x
	GT100GCX_Options.posY = y
	GT100GCX_Options.relP = relativePoint
end

function GT100GCX_SearchFrameToggle()
	if GT100GCXFrame_search:IsShown() then
		GT100GCX_SearchFrameHide()
		GT100GCX_CreateGUItab()
		GT100GCX_FillList()
		GT100GCX_UpdateStatusFrame()
	else
		GT100GCX_SearchFrameShow()
	end
end

function GT100GCX_SearchFrameHide()
	SearchQuery = nil
	GT100GCXFrame_searchBox:SetText("")
	GT100GCXFrame_search:Hide()
	GT100GCXFrame_sort:SetPoint("TOPLEFT", "GT100GCXFrame_headline", "BOTTOMLEFT", 150, 0)
	GT100GCXFrame_groups:SetHeight(330)
	GT100GCXFrame:SetHeight(400)
	GT100GCX_Options.searchshow = false
end

function GT100GCX_SearchFrameShow()
	GT100GCXFrame_search:Show()
	GT100GCXFrame_sort:SetPoint("TOPLEFT", "GT100GCXFrame_search", "BOTTOMLEFT", 0, 0)
	GT100GCXFrame_groups:SetHeight(360)
	GT100GCXFrame:SetHeight(430)
	GT100GCX_Options.searchshow = true
end

function GT100GCX_Search()
	SearchQuery = GT100GCXFrame_searchBox:GetText()
	GT100GCX_CreateGUItab()
	GT100GCX_FillList()
	GT100GCX_UpdateStatusFrame()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function GT100GCX_UpdateStatusFrame()
	local w = 0 -- magik w trip trick

	local factionUnknown     = GUIStatusTable.Zero or 0
	local factionAlliance    = GUIStatusTable.Alliance or 0
	local factionHorde       = GUIStatusTable.Horde or 0
	local summaryFaction     = factionUnknown + factionAlliance + factionHorde
	local summaryPlayerCount = GUIStatusTable.player or 0

	if factionUnknown     == 0 then factionUnknown = "-" end
	if factionAlliance    == 0 then factionAlliance = "-" end
	if factionHorde       == 0 then factionHorde = "-" end
	if summaryFaction     == 0 then summaryFaction = "-" end
	if summaryPlayerCount == 0 then summaryPlayerCount = "-" end

	local txt = summaryFaction.." |cff"..color.Orange.hex.."("..summaryPlayerCount..")|r  ["
	GT100GCXFrame_StatusLineSummaryText:SetText(txt)
	w = w + GT100GCXFrame_StatusLineSummaryText:GetWidth()

	GT100GCXFrame_StatusLineFactionUnknownText:SetText("?: "..factionUnknown.."  ")
	w = w + GT100GCXFrame_StatusLineFactionUnknownText:GetWidth()

	GT100GCXFrame_StatusLineFactionAllianceText:SetText(": "..factionAlliance.."  ")
	w = w + GT100GCXFrame_StatusLineFactionAllianceText:GetWidth()

	GT100GCXFrame_StatusLineFactionHordeText:SetText(": "..factionHorde.."]    -    ")
	w = w + GT100GCXFrame_StatusLineFactionHordeText:GetWidth()

	w = w + 24 -- alliance/horde img

	for i=1, #classesINT do
		local text = GUIStatusTable[classesINT[i]] or "-"
		local name = "GT100GCXFrame_StatusLine"..classesINT[i]
		if not _G[name] then
			local parent
			if i == 1 then
				parent = GT100GCXFrame_StatusLineFactionHordeText
			else
				parent = _G["GT100GCXFrame_StatusLine"..classesINT[i-1].."Text"]
			end
			local f = GT100GCXFrame_StatusLine
			local t = f:CreateTexture(name)
			t:SetPoint("LEFT", parent, "RIGHT", 0, -2)
			t:SetWidth(12)
			t:SetHeight(12)
			t:SetTexture(classimg)
			t:SetTexCoord(classes[classesINT[i]].icon[1], classes[classesINT[i]].icon[2], classes[classesINT[i]].icon[3], classes[classesINT[i]].icon[4])

			local txt = f:CreateFontString(name.."Text", "OVERLAY", "GameFontNormalSmall")
			txt:SetPoint("LEFT", t, "RIGHT", 0, 2)
			txt:SetTextColor(color.White.rgb[1], color.White.rgb[2], color.White.rgb[3], 1)
			t:Show()
		end

		_G[name.."Text"]:SetText(": "..text.."  ")
		w = w + _G[name.."Text"]:GetWidth()
	end

	w = w + (#classesINT * 12)
	GT100GCXFrame_StatusLineSummaryText:SetPoint("LEFT", GT100GCXFrame_StatusLine, "LEFT", ((GT100GCXFrame:GetWidth()-w)/2), 0)
end

function GT100GCX_UpdateSortButtons()
	-- GUItabSortBy    - main data table sort by: 1 = pos/GUID | 2 = name | 3 = faction | 4 = class | 5 = realm | 6 = date
	-- GUItabSortOrder - main data table sort order: 0 = asc | 1 = desc
	-- arrows
	local arrow
	local arrow2
	for i=1, 6 do
		if i == 1 then -- 1 = pos/GUID
			arrow = GT100GCXFrame_BrowseSortPosArrowArrow
			arrow2 = GT100GCXFrame_BrowseSortGuidArrowArrow
		elseif i == 2 then -- 2 = name
			arrow = GT100GCXFrame_BrowseSortNameArrowArrow
		elseif i == 3 then -- 3 = faction
			arrow = GT100GCXFrame_BrowseSortFactionArrowArrow
		elseif i == 4 then -- 4 = class
			arrow = GT100GCXFrame_BrowseSortClassArrowArrow
		elseif i == 5 then -- 5 = realm
			arrow = GT100GCXFrame_BrowseSortRealmArrowArrow
		elseif i == 6 then -- 6 = date
			arrow = GT100GCXFrame_BrowseSortDateArrowArrow
		end
		if GUItabSortBy == i then
			if GUItabSortOrder == 1 then arrow:SetTexCoord(0.5625, 0, 1, 0) end
			if GUItabSortOrder == 0 then arrow:SetTexCoord(0, 0.5625, 0, 1) end
			if i == 1 then
				if GUItabSortOrder == 1 then arrow2:SetTexCoord(0.5625, 0, 1, 0) end
				if GUItabSortOrder == 0 then arrow2:SetTexCoord(0, 0.5625, 0, 1) end
			end
		else
			arrow:SetTexCoord(0, 0, 0, 0)
			if i == 1 then
				arrow2:SetTexCoord(0, 0, 0, 0)
			end
		end
	end
end

function GT100GCX_InitFrame()
	-- search toggle
	if GT100GCX_Options.searchshow then
		GT100GCX_SearchFrameShow()
	else
		GT100GCX_SearchFrameHide()
	end

	-- sort, position & localize sort buttons
	GT100GCX_UpdateSortButtons()

	GT100GCXFrame_BrowseSortPosText:ClearAllPoints()
	GT100GCXFrame_BrowseSortPosText:SetPoint("RIGHT", -12, 0)

	GT100GCXFrame_BrowseSortPosText:SetText(L["Pos"])
	GT100GCXFrame_BrowseSortNameText:SetText(L["Name"])
	GT100GCXFrame_BrowseSortRealmText:SetText(L["Realm"])
	GT100GCXFrame_BrowseSortGuidText:SetText("GUID")
	GT100GCXFrame_BrowseSortDateText:SetText(L["Date"])

	-- localize other buttons
	GT100GCXFrame_headline_ButtonOptions:SetText(L["Options"])
	GT100GCXFrame_headline_ButtonOptions:SetWidth( (tonumber(GT100GCXFrame_headline_ButtonOptions:GetTextWidth())+20) )

	GT100GCXFrame_searchBox_ClearButton:SetText(L["Clear"])
	GT100GCXFrame_searchBox_ClearButton:SetWidth( (tonumber(GT100GCXFrame_searchBox_ClearButton:GetTextWidth())+20) )

	GT100GCXFrame_searchBox_SearchText:SetText(L["search"]..":")
	GT100GCXFrame_RealmTitleTxt:SetText(L["Realm"]..":")

	-- scroll knob tweak
	GT100GCXFrame_ScrollFrameScrollBarThumbTexture:SetWidth(16)
	GT100GCXFrame_ScrollFrameScrollBarThumbTexture:SetHeight(16)
	GT100GCXFrame_ScrollFrameScrollBarThumbTexture:SetTexCoord(0.25, 0.75, 0.25, 0.75)

	GT100GCXFrame_RealmScrollFrameScrollBarThumbTexture:SetWidth(16)
	GT100GCXFrame_RealmScrollFrameScrollBarThumbTexture:SetHeight(16)
	GT100GCXFrame_RealmScrollFrameScrollBarThumbTexture:SetTexCoord(0.25, 0.75, 0.25, 0.75)

--bordertest
--GT100GCXFrame_headlineTempColor:SetTexture(  0.4, 0.4, 0.3, 1)--(0.4, 0.4, 0.4, 1)
--GT100GCXFrame_groupsTempColor:SetTexture(    0.2, 0.5, 0.1, 1)--(0.4, 0.4, 0.4, 1)
--GT100GCXFrame_searchTempColor:SetTexture(    0.7, 0.6,   1, 1)--(0.4, 0.4, 0.4, 1)
--GT100GCXFrame_sortTempColor:SetTexture(        1, 0.8,   0, 1)--(0.4, 0.4, 0.4, 1)
--GT100GCXFrame_ListViewTempColor:SetTexture(  0.5, 0.5, 0.3, 1)--(0.4, 0.4, 0.4, 1)
--GT100GCXFrame_StatusLineTempColor:SetTexture(  1, 0.2, 0.6, 1)--(0.4, 0.4, 0.4, 1)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function GT100GCX_OnEvent()
	-- -------------------------------------------------------------------------------------------------------------------
	-- combatlog scan
	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		-- argx vs timestamp, event, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags
		-- timestamp = arg1
		-- event     = arg2
		-- srcGUID   = arg3
		-- srcName   = arg4
		-- srcFlags  = arg5
		-- dstGUID   = arg6
		-- dstName   = arg7
		-- dstFlags  = arg8

		-- no srcName = return to sender
		if not arg4 then return end
		-- no srcGUID = return to sender
		if not arg3 then return end
		-- no srcFlags = return to sender
		if not arg5 then return end
		-- it's me? = return to sender
		if arg4 == playerName then return end
		-- only player units (0x00000400 / COMBATLOG_OBJECT_TYPE_PLAYER)
		if bit_band(arg5, 0x00000400) == 0x00000400 then
			-- arg4 (srcName) returns 'name' or 'name-server' (in cross-server battlegrounds) - GT100GCX wanna know
			local name = arg4
			local realm = playerRealm
			if string_find(arg4, "-", 1, true) then              -- think it's better to do a plain string.find first
				if GT100GCX_Options.realmscan then return end
				_, _, name, realm = string_find(arg4, "(.-)-(.*)") -- and do the regex only if arg4 (srcName) is not from our server
			end
			GT100GCX_SaveData(L["COMBATLOG"], name, realm, arg3)
		end
	-- -------------------------------------------------------------------------------------------------------------------
	-- combat
	elseif event == "PLAYER_REGEN_DISABLED" then
		inCombat = true
		if not GT100GCX_Options.incombatscan then
			GT100GCX:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			GT100GCX:UnregisterEvent("RAID_ROSTER_UPDATE")
			GT100GCX:UnregisterEvent("PARTY_MEMBERS_CHANGED")
			GT100GCX:UnregisterEvent("CHAT_MSG_CHANNEL")
			GT100GCX:UnregisterEvent("CHAT_MSG_CHANNEL_NOTICE_USER")
		end
	-- -------------------------------------------------------------------------------------------------------------------
	-- out-of-combat
	elseif event == "PLAYER_REGEN_ENABLED" then
		inCombat = false
		if not GT100GCX_Options.incombatscan then
			if GT100GCX_Options.combatlogscan then
				GT100GCX:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			end
			if GT100GCX_Options.groupscan then
				GT100GCX:RegisterEvent("PARTY_MEMBERS_CHANGED")
				GT100GCX:RegisterEvent("RAID_ROSTER_UPDATE")
			end
			if GT100GCX_Options.chatscan then
				GT100GCX:RegisterEvent("CHAT_MSG_CHANNEL")
				GT100GCX:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE_USER")
			end
		end
	-- -------------------------------------------------------------------------------------------------------------------
	-- chat scan
	elseif event == "CHAT_MSG_CHANNEL" then
		GT100GCX_ChatScan(event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12)
	elseif event == "CHAT_MSG_CHANNEL_NOTICE_USER" then
		GT100GCX_ChatScan(event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12)
	-- -------------------------------------------------------------------------------------------------------------------
	-- raid scan
	elseif event == "RAID_ROSTER_UPDATE" then
		local raidmembers = GetNumRaidMembers()
		if rosterhack ~= raidmembers then -- avoid execute this block if raidmembers are just moved around
			for i=1, raidmembers do
				local name, realm = UnitName("raid"..i)
				local guid = UnitGUID("raid"..i)
				GT100GCX_SaveData(L["RAID"], name, realm, guid)
			end
			rosterhack = raidmembers
		end
	-- -------------------------------------------------------------------------------------------------------------------
	-- group scan
	elseif event == "PARTY_MEMBERS_CHANGED" then
		if GetNumRaidMembers() == 0 then -- avoid execute this block if we are in raid
			for i=1, GetNumPartyMembers() do
				local name, realm = UnitName("party"..i)
				local guid = UnitGUID("party"..i)
				GT100GCX_SaveData(L["PARTY"], name, realm, guid)
			end
		end
	-- -------------------------------------------------------------------------------------------------------------------
	elseif event == "PLAYER_ENTERING_WORLD" then
		playerClass = select(2, UnitClass("player"))
		playerFaction = UnitFactionGroup("player")
		GT100GCX_SavePlayerData()
		GT100GCX_LDBcheck()
		GT100GCX:UnregisterEvent("PLAYER_ENTERING_WORLD")
	-- -------------------------------------------------------------------------------------------------------------------
	elseif event == "ADDON_LOADED" then
		if arg1 == "GT100GCX" then
			GT100GCX_InitOptions()
			GT100GCX_CreateInterfaceOptions()
			GT100GCX_InitFrame()
			tinsert(UISpecialFrames, "GT100GCXFrame")
			GameTooltip:HookScript("OnTooltipSetUnit", GT100GCX_OnTooltipSetUnit)
			GT100GCX:UnregisterEvent("ADDON_LOADED")
		end
	end
	-- -------------------------------------------------------------------------------------------------------------------
end
-- ---------------------------------------------------------------------------------------------------------------------

GT100GCX:RegisterEvent("ADDON_LOADED")
GT100GCX:RegisterEvent("PLAYER_ENTERING_WORLD")
GT100GCX:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED") -- combatlog scan
GT100GCX:RegisterEvent("RAID_ROSTER_UPDATE")          -- raid scan
GT100GCX:RegisterEvent("PARTY_MEMBERS_CHANGED")       -- group scan
GT100GCX:RegisterEvent("CHAT_MSG_CHANNEL")            -- chat scan
GT100GCX:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE_USER")-- chat scan
GT100GCX:RegisterEvent("PLAYER_REGEN_DISABLED")       -- combat
GT100GCX:RegisterEvent("PLAYER_REGEN_ENABLED")        -- out-of-combat
GT100GCX:SetScript("OnEvent", GT100GCX_OnEvent)       -- start the Mystical Machine